import 'dart:math';

import 'package:flutter/rendering.dart';

class ArcPainter extends CustomPainter {
  ArcPainter({
    /// We assume that target space will be square.
    double fullWidth,
    Offset center,
    double radius,
    this.offset,
    this.angle,
    this.color,
  })  : center = center * fullWidth,
        radius = radius * fullWidth;

  /// Relative center (0.0~1.0)
  final Offset center;

  /// Relative radius (0.0~1.0)
  final double radius;

  /// Relative offset (0.0~1.0)
  final double offset;

  /// Relative angle (0.0~1.0)
  final double angle;

  /// Arc color
  final Color color;

  @override
  void paint(Canvas canvas, Size size) {
    final paint = Paint()
      ..style = PaintingStyle.fill
      ..color = color
      ..isAntiAlias = true;

    canvas.drawArc(
      Rect.fromCircle(center: center, radius: radius),
      offset * 2 * pi - pi / 2,
      angle * 2 * pi,
      true,
      paint,
    );
    canvas.save();
    canvas.restore();
  }

  @override
  bool shouldRepaint(ArcPainter oldDelegate) {
    return oldDelegate.center != center ||
        oldDelegate.radius != radius ||
        oldDelegate.offset != offset ||
        oldDelegate.angle != angle;
  }
}
