import 'package:flutter/material.dart';

class FlatListBox extends StatelessWidget {
  const FlatListBox({
    @required this.children,
    this.padding = const EdgeInsets.symmetric(horizontal: 24.0),
    this.color,
    this.onPressed,
    this.alignment = MainAxisAlignment.spaceBetween,
    this.minHeight = 64.0,
    this.hasSeparator = true,
  });

  final List<Widget> children;
  final EdgeInsetsGeometry padding;
  final Color color;
  final VoidCallback onPressed;
  final MainAxisAlignment alignment;
  final double minHeight;
  final bool hasSeparator;

  @override
  Widget build(BuildContext context) {
    Widget current = Row(
      mainAxisAlignment: alignment,
      children: children,
    );

    current = ConstrainedBox(
      constraints: BoxConstraints(minHeight: minHeight),
      child: current,
    );

    if (onPressed != null) {
      current = FlatButton(
        padding: padding,
        textColor: Theme.of(context).textTheme.body1.color,
        child: current,
        onPressed: onPressed,
      );
    } else {
      current = Padding(
        padding: padding,
        child: current = current,
      );
    }

    if (hasSeparator) {
      current = Container(
        decoration: BoxDecoration(
          color: color ?? Theme.of(context).cardColor,
          border: Border(
            bottom: BorderSide(
              color: Theme.of(context).dividerColor,
            ),
          ),
        ),
        child: current,
      );
    } else {
      current = Container(
        color: color,
        child: current,
      );
    }
    return current;
  }
}

class FlatListWidgetsBox extends StatelessWidget {
  final Widget leadingWidget;
  final Widget mainWidget;
  final Widget trailingWidget;
  final VoidCallback onPressed;
  final EdgeInsetsGeometry padding;
  final double minHeight;
  final bool hasSeparator;

  const FlatListWidgetsBox({
    this.leadingWidget,
    @required this.mainWidget,
    this.trailingWidget,
    @required this.onPressed,
    this.padding = const EdgeInsets.symmetric(horizontal: 24.0),
    this.minHeight = 64.0,
    this.hasSeparator = true,
  }) : assert(mainWidget != null);

  @override
  Widget build(BuildContext context) {
    return FlatListBox(
      children: <Widget>[
        if (leadingWidget != null) ...[
          Padding(
            padding: const EdgeInsets.only(right: 8.0),
            child: leadingWidget,
          ),
        ],
        Expanded(child: mainWidget),
        if (trailingWidget != null) ...[
          trailingWidget,
        ],
      ],
      padding: padding,
      onPressed: onPressed,
      minHeight: minHeight,
      hasSeparator: hasSeparator,
    );
  }
}

class FlatListIconBox extends StatelessWidget {
  final IconData _iconData;
  final String _title;
  final VoidCallback onPressed;

  const FlatListIconBox(
    this._iconData,
    this._title, {
    @required this.onPressed,
  });

  @override
  Widget build(BuildContext context) {
    return FlatListWidgetsBox(
      leadingWidget: Icon(_iconData),
      mainWidget: Text(
        _title,
        style: Theme.of(context).textTheme.subhead,
      ),
      onPressed: onPressed,
    );
  }
}
