import 'package:flutter/material.dart';

class HorizontalListView extends StatelessWidget {
  final double aspectRatio;
  final double viewportFraction;
  final double spacing;
  final int itemCount;
  final IndexedWidgetBuilder itemBuilder;

  const HorizontalListView({
    Key key,
    this.aspectRatio = 1.0,
    this.viewportFraction = 1.0,
    this.spacing = 16.0,
    @required this.itemCount,
    @required this.itemBuilder,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (BuildContext context, BoxConstraints constraints) {
        final double itemWidth = constraints.maxWidth * viewportFraction;
        final double itemHeight = itemWidth * aspectRatio;
        return Container(
          height: itemHeight,
          child: ListView.builder(
            physics: const PageScrollPhysics(),
            controller: PageController(viewportFraction: viewportFraction),
            scrollDirection: Axis.horizontal,
            itemExtent: itemWidth,
            itemCount: itemCount,
            itemBuilder: (context, index) {
              return Padding(
                padding: itemCount == index + 1
                    ? EdgeInsets.only(left: spacing, right: spacing)
                    : EdgeInsets.only(left: spacing),
                child: itemBuilder(context, index),
              );
            },
          ),
        );
      },
    );
  }
}
